@@ -1,9 +1,12 @@
 import json
-from terraform_compliance.common.helper import seek_key_in_dict, flatten_list, dict_merge, Match
+from terraform_compliance.common.helper import seek_key_in_dict, flatten_list, Match, merge_dicts, remove_constant_values
 import sys
 from copy import deepcopy
+from radish.utils import console_write
 from terraform_compliance.common.defaults import Defaults
 from terraform_compliance.extensions.cache import Cache
+from terraform_compliance.common.exceptions import TerraformComplianceInternalFailure
+
 
 class TerraformParser(object):
     def __init__(self, filename, parse_it=True):
@@ -18,8 +21,9 @@ class TerraformParser(object):
         :return: None
         '''
         self.supported_terraform_versions = (
-            '0.12', # This is here because this tuple must have multiple values.
-            '0.12.'
+            '0.12.',
+            '0.13.',
+            '0.14.'
         )
         self.supported_format_versions = ['0.1']
 
@@ -34,6 +38,7 @@ class TerraformParser(object):
         self.configuration = dict(resources={}, variables={})
         self.file_type = "plan"
         self.resources_raw = {}
+        self.type_to_after_unknown_properties = {}
         self.parse_it = parse_it
 
         if parse_it:
@@ -130,7 +135,8 @@ class TerraformParser(object):
             change = resource.get('change', {})
             actions = change.get('actions', [])
             if actions != ['delete']:
-                resource['values'] = dict_merge(change.get('after', {}), change.get('after_unknown', {}))
+                resource['values'] = change.get('after', {}) # dict_merge(change.get('after', {}), change.get('after_unknown', {}))
+                self.remember_after_unknown(resource, change.get('after_unknown', {}))
                 if 'change' in resource:
                     del resource['change']
 
@@ -142,6 +148,38 @@ class TerraformParser(object):
         if self.parse_it:
             self.cache.set('resources', self.resources)
 
+    '''
+    Creates a map of resource type to after_unknown values
+    These can be used in given step "resource that supports x"
+    
+    Note: This function may be extended to capture 'after' values as well. That would require flattening the multi level
+    dictionaries in resource
+    '''
+    def remember_after_unknown(self, resource, after_unknown):
+        # get type
+        resource_type = resource.get('type', '')
+
+        # if resource doesn't have type field, try to extract it from address (ideally, this if never evaluates true)
+        if not resource_type and 'address' in resource and resource['address']:
+            parsed_address = resource.get('address').split('.')
+            if parsed_address != 'module':
+                resource_type = parsed_address[0]
+            elif len(parsed_address) >= 3:
+                resource_type = parsed_address[2]
+            else:
+                return
+
+        # get after_unknown values
+
+        # need to merge because which values are in after_unknown may change from instance to instance
+        # merging rule: if field not in map, add it
+        if resource_type not in self.type_to_after_unknown_properties:
+            self.type_to_after_unknown_properties[resource_type] = after_unknown
+        else:
+            for key, value in after_unknown.items():
+                if key not in self.type_to_after_unknown_properties[resource_type]:
+                    self.type_to_after_unknown_properties[resource_type][key] = value
+
     def _parse_configurations(self):
         '''
         Assigns all configuration related data defined in the terraform plan. This is mostly used for
@@ -160,8 +198,6 @@ class TerraformParser(object):
         # Resources
         self.configuration['resources'] = {}
 
-        resources = []
-
         # root resources
         resources = self.raw.get('configuration', {}).get('root_module', {}).get('resources', [])
 
@@ -169,9 +205,11 @@ class TerraformParser(object):
         for module in seek_key_in_dict(self.raw.get('configuration', {}).get('root_module', {}).get("module_calls", {}), "module"):
             resources += module.get('module',{}).get("resources", [])
 
+        remove_constant_values(resources)
         for resource in resources:
             if self.is_type(resource, 'data'):
                 self.data[resource['address']] = resource
+
             else:
                 self.configuration['resources'][resource['address']] = resource
 
@@ -253,19 +291,13 @@ class TerraformParser(object):
 
                     if ref_type not in self.resources[target_resource]['values']:
                         self.resources[target_resource]['values'][ref_type] = []
-                        self.resources[target_resource]['values'][ref_type].append(resource)
-                        self.resources[target_resource][Defaults.r_mount_ptr][parameter] = ref_type
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr][parameter] = source
-                        target_set = set(self.resources[target_resource][Defaults.r_mount_addr_ptr_list])
-                        source_set = set(source)
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list] = list(target_set | source_set)
-                    else:
-                        self.resources[target_resource]['values'][ref_type].append(resource)
-                        self.resources[target_resource][Defaults.r_mount_ptr][parameter] = ref_type
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr][parameter] = source
-                        target_set = set(self.resources[target_resource][Defaults.r_mount_addr_ptr_list])
-                        source_set = set(source)
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list] = list(target_set | source_set)
+
+                    self.resources[target_resource]['values'][ref_type].append(resource)
+                    self.resources[target_resource][Defaults.r_mount_ptr][parameter] = ref_type
+                    self.resources[target_resource][Defaults.r_mount_addr_ptr][parameter] = source
+                    target_set = set(self.resources[target_resource][Defaults.r_mount_addr_ptr_list])
+                    source_set = set(source)
+                    self.resources[target_resource][Defaults.r_mount_addr_ptr_list] = list(target_set | source_set)
 
                     if parameter not in self.resources[source_resource]['values']:
                         self.resources[source_resource]['values'][parameter] = target_resource
@@ -309,7 +341,7 @@ class TerraformParser(object):
         :return:
         '''
         self.resources_raw = deepcopy(self.resources)
-        invalid_references = ('var.')
+        invalid_references = ('var.', 'each.')
 
         # This section will link resources found in configuration part of the plan output.
         # The reference should be on both ways (A->B, B->A) since terraform sometimes report these references
@@ -318,18 +350,76 @@ class TerraformParser(object):
             if 'expressions' in self.configuration['resources'][resource]:
                 ref_list = {}
                 for key, value in self.configuration['resources'][resource]['expressions'].items():
-                    if 'references' in value:
-                        for ref in value['references']:
-                            if not ref.startswith(invalid_references):
-                                if key not in ref_list:
-                                    ref_list[key] = self._find_resource_from_name(ref)
-                                else:
-                                    ref_list[key].extend(self._find_resource_from_name(ref))
+                    references = seek_key_in_dict(value, 'references') if isinstance(value, (dict, list)) else []
+
+                    valid_references = []
+                    for ref in references:
+                        if isinstance(ref, dict) and ref.get('references'):
+                            valid_references = [r for r in ref['references'] if not r.startswith(invalid_references)]
+
+                    for ref in valid_references:
+                        # if ref is not in the correct format, handle it
+                        if len(ref.split('.')) < 3 and ref.startswith('module'):
+                            
+                            # Using for_each and modules together may introduce an issue where the plan.out.json won't include the necessary third part of the reference
+                            # It is partially resolved by mounting the reference to all instances belonging to the module
+                            if 'for_each_expression' in self.configuration['resources'][resource]:
+
+                                # extract source resources
+                                assumed_source_resources = [k for k in self.resources.keys() if k.startswith(resource)]
+                                # extract for_each keys
+                                assumed_for_each_keys = [k[len(resource):].split('.')[0] for k in assumed_source_resources]
+                                # combine ref with for each keys
+                                assumed_refs = ['{}{}'.format(ref, key) for key in assumed_for_each_keys]
+                                # get all the resources that start with updated ref
+                                ambigious_references = []
+                                for r in self.resources.keys():
+                                    for assumed_ref in assumed_refs:
+                                        if r.startswith(assumed_ref):
+                                            if key in ref_list:
+                                                ref_list[key].append(r)
+                                            else:
+                                                ref_list[key] = [r]
+
+                                            ambigious_references.append(r)
+
+                                # throw a warning
+                                defaults = Defaults()
+                                console_write('{} {}: {}'.format(defaults.warning_icon,
+                                       defaults.warning_colour('WARNING (Mounting)'),
+                                       defaults.info_colour('The reference "{}" in resource {} is ambigious.'
+                                        ' It will be mounted to the following resources:').format(ref, resource)))
+                                for i, r in enumerate(ambigious_references, 1):
+                                    console_write(defaults.info_colour('{}. {}'.format(i, r)))
+                                
+                            # if the reference can not be resolved, warn the user and continue.
+                            else: 
+                                console_write('{} {}: {}'.format(Defaults().warning_icon,
+                                       Defaults().warning_colour('WARNING (Mounting)'),
+                                       Defaults().info_colour('The reference "{}" in resource {} is ambigious. It will not be mounted.'.format(ref, resource))))
+                                continue
+                        elif key not in ref_list:
+                            ref_list[key] = self._find_resource_from_name(ref)
+                        else:
+                            ref_list[key].extend(self._find_resource_from_name(ref))
+
+                    # This is where we synchronise constant_value in the configuration section with the resource
+                    # for filling up the missing elements that hasn't been defined in the resource due to provider
+                    # implementation.
+                    target_resource = [t for t in [self.resources.get(resource, {}).get('address')] if t is not None]
+                    if not target_resource:
+                        target_resource = [k for k in self.resources.keys() if k.startswith(resource)]
+
+                    for t_r in target_resource:
+                        if type(value) is type(self.resources[t_r]['values'].get(key)) and self.resources[t_r]['values'].get(key) != value:
+                            if isinstance(value, (list, dict)):
+                                merge_dicts(self.resources[t_r]['values'][key], value)
 
                 if ref_list:
-                    ref_type = self.configuration['resources'][resource]['expressions'].get('type',
-                                                                                            {}).get('constant_value',
-                                                                                                    {})
+                    ref_type = self.configuration['resources'][resource]['expressions'].get('type', {})
+
+                    if 'references' in ref_type:
+                        ref_type = resource.split('.')[0]
 
                     if not ref_type and not self.is_type(resource, 'data'):
                         resource_type, resource_id = resource.split('.')
@@ -377,17 +467,23 @@ class TerraformParser(object):
             self._parse_variables()
             self._parse_configurations()
 
-        cache = self.cache.get('mounted_resources') if self.parse_it else None
+        cache_mounted_resources = self.cache.get('mounted_resources') if self.parse_it else None
+        cache_raw_resources = self.cache.get('resources_raw') if self.parse_it else None
+        cache_type_to_after_unknown_properties = self.cache.get('type_to_after_unknown_properties') if self.parse_it else None
 
-        if cache:
+        if cache_mounted_resources and cache_raw_resources:
             # print('Read from cache, instead of re-mounting.')
-            self.resources = cache
+            self.resources = cache_mounted_resources
+            self.resources_raw = cache_raw_resources
+            self.type_to_after_unknown_properties = cache_type_to_after_unknown_properties
         else:
             # print('Building cache for mounted resources at {}'.format(Defaults.cache_dir))
             self._mount_references()
 
             if self.parse_it:
                 self.cache.set('mounted_resources', self.resources)
+                self.cache.set('resources_raw', self.resources_raw)
+                self.cache.set('type_to_after_unknown_properties', self.type_to_after_unknown_properties)
 
         self._distribute_providers()
 
